unit Unit1;

interface

{x$define Reporting}
{x$define CodeSite}


uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants, System.Classes, Vcl.Graphics,
  Vcl.Controls, Vcl.Forms, Vcl.Dialogs, VclTee.TeeGDIPlus, VCLTee.TeEngine, TeCanvas,
  Vcl.StdCtrls, Vcl.Samples.Spin,
  {$if defined(Reporting)}
  ppPrnabl, ppClass, ppCtrls, ppBands, ppCache, ppDesignLayer, ppParameter,
  ppComm, ppRelatv, ppProd, ppReport, ppEndUsr,
  {$endif}
  VCLTee.Series, Vcl.ExtCtrls, VCLTee.TeeProcs, VCLTee.Chart, Vcl.ComCtrls;


type
  TForm1 = class(TForm)
    PageControl1: TPageControl;
    tsChart: TTabSheet;
    tsGenImage: TTabSheet;
    Chart1: TChart;
    Series1: TBarSeries;
    pnlChartHeader: TPanel;
    pnlImageHeader: TPanel;
    imgChart: TImage;
    procedure tsGenImageShow(Sender: TObject);
  private
    {$if defined(Reporting)}
    FppReport: TppReport;
    FppImage: TppImage;
    {$endif}
    { Private declarations }
    procedure UpdateImage;
    {$if defined(Reporting)}
    procedure PrintReport;
    procedure UpdateReportImage;
    procedure btnCreateReport(Sender: TObject);
    {$endif}
  public
    { Public declarations }
    constructor Create(aOwner: TComponent); override;
  end;

var
  Form1: TForm1;

implementation

{$R *.dfm}

{$if defined(CodeSite)}
uses CodeSiteLogging;
{$endif}

procedure AssignChartToGraphic(aChart: TChart; const aRect: TRect; aGraphic: TGraphic; aTransparent: boolean = false);
var
  lHoldShadow: boolean;
  I: Integer;
  lBoolArray: array of boolean;
  lHoldBevel: TBevelCut;
  lHoldColor: TColor;
  lHoldGradientVisible: boolean;
  lMetafile: TMetafile;
  {$ifopt C+}
  lCanvasClassName: string;
  {$endif}
  lBmp: TBitmap;
  lCanvasClass: TCanvas3DClass;
  lHoldHover: Boolean;
begin
  {$ifopt C+}
  lCanvasClassName := aChart.Canvas.ClassName;
  {$endif}
  lCanvasClass := TCanvas3DClass(aChart.Canvas.ClassType);
  if lCanvasClass <> TTeeCanvas3D then
    aChart.Canvas := TTeeCanvas3D.Create;

  lHoldGradientVisible := aChart.Gradient.Visible;
  lHoldColor := aChart.Color;
  lHoldBevel := aChart.BevelOuter;
  lHoldShadow := aChart.Shadow.Visible;
  lHoldHover := aChart.Hover.Visible;

  try
    SetLength(lBoolArray, aChart.SeriesCount);
    for I := 0 to aChart.SeriesCount - 1 do    // Iterate
      if aChart.Series[I] is TPieSeries then
      begin
        lBoolArray[I] := TPieSeries(aChart.Series[I]).Circled;
        TPieSeries(aChart.Series[I]).Circled := True;
      end
      else
        lBoolArray[I] := True;


    aChart.Gradient.Visible  := false;
    if not aTransparent then
      aChart.Color := clWhite
    else
      aChart.Color := VCL.Graphics.clNone; //clWhite;
    aChart.BevelOuter := bvNone;
    aChart.BevelInner := bvNone;
    aChart.Shadow.Visible := False;
    aChart.Hover.Visible := false;

    if aGraphic is TMetafile then
    begin
      lMetafile := aChart.TeeCreateMetafile(false, aRect);
      try
        aGraphic.Assign(lMetafile);
      finally
        lMetafile.Free;
      end;
    end
    else
    begin
      lBmp := aChart.TeeCreateBitmap(clWhite, aRect);
      try
        aGraphic.Assign(lBmp);
      finally
        lBmp.Free;
      end;
    end;
  finally
    aChart.Hover.Visible := lHoldHover;
    aChart.Shadow.Visible := lHoldShadow;
    aChart.Gradient.Visible := lHoldGradientVisible;
    aChart.Color := lHoldColor;
    aChart.BevelOuter := lHoldBevel;
    for I := 0 to aChart.SeriesCount - 1 do    // Iterate
      if aChart.Series[I] is TPieSeries then
        TPieSeries(aChart.Series[I]).Circled := lBoolArray[I];
    if (lCanvasClass <> aChart.Canvas.ClassType) then
      aChart.Canvas := lCanvasClass.Create;
  end;

  {$ifopt C+}
  assert(SameText(lCanvasClassName, aChart.Canvas.ClassName),
    Format('Chart canvas should be %s but is %s', [lCanvasClassName, aChart.Canvas.ClassName]));
  {$endif}
end;


constructor TForm1.Create(aOwner: TComponent);
var
  lts: TTabSheet;
  lbtn: TButton;
begin
  inherited;
  Series1.FillSampleValues(20);

  pnlChartHeader.Font := Chart1.Title.Font;
  pnlChartHeader.Caption := Chart1.Title.Caption;

  pnlImageHeader.Font := Chart1.Title.Font;
  pnlImageHeader.Caption := Chart1.Title.Caption;

  {$if defined(Reporting)}
  lts := TTabSheet.Create(self);
  lts.PageControl := PageControl1;
  lts.Caption := 'Report';

  lbtn := TButton.Create(self);
  lbtn.Parent := lts;
  lbtn.Caption := 'Create Report';
  lbtn.OnClick := btnCreateReport;

  FppReport := TpPReport.Create(self);

  FppImage := TppImage.Create(self);
  FppImage.Band := FppReport.DetailBand;
  FppImage.Left := 0;
  FppImage.spWidth := FppReport.DetailBand.spWidth;

  FppReport.DetailBand.spHeight := (FppImage.spWidth div 2) + 10;
  FppImage.spHeight := FppImage.spWidth div 2;
  {$endif}
end;

{$if defined(Reporting)}
procedure TForm1.PrintReport;
begin
  UpdateReportImage;

//  ppDesigner1.Report := FppReport;
//  ppDesigner1.ShowModal;

  FppReport.Print;
end;

procedure TForm1.btnCreateReport(Sender: TObject);
begin
  PrintReport;
end;

procedure TForm1.UpdateReportImage;
var
  lBmp: TBitmap;
  lRect: TRect;
  lFactor: double;
begin
  lBmp := TBitmap.Create;
  try
    lFactor := 1;
    lBmp.SetSize(ROund(lFactor * FppImage.spWidth), ROund(lFactor * FppImage.spHeight));
    lRect := Rect(1, 1,  ROund(lFactor * FppImage.spWidth), ROund(lFactor * FppImage.spHeight));

    AssignChartToGraphic(Chart1, lRect, lBmp, false);

    FppImage.Picture.Graphic := lBmp;

    {$if defined(CodeSite)}
    CodeSite.Send('lRect', lRect);
    CodeSite.Send('lBmp', lBmp);
    {$endif}
  finally
    lBmp.Free;
  end;
end;
{$endif}

procedure TForm1.tsGenImageShow(Sender: TObject);
begin
  UpdateImage;
end;

procedure TForm1.UpdateImage;
var
  lBmp: TBitmap;
  lRect: TRect;
begin
  lBmp := TBitmap.Create;
  try
    lBmp.SetSize(imgChart.Width, imgChart.Height);
    lRect := Rect(1, 1,  imgChart.Width, imgChart.Height);

    AssignChartToGraphic(Chart1, lRect, lBmp, false);

    imgChart.Picture.Graphic := lBmp;
    {$if defined(CodeSite)}
    CodeSite.Send('lRect', lRect);
    CodeSite.Send('lBmp', lBmp);
    {$endif}
  finally
    lBmp.Free;
  end;
end;

end.
